#!/bin/sh
#----------------------------------------------------------------------------
# /sbin/mdadm-event-handler.sh
#
# Creation   :  2008-08-12 Arno van Amersfoort
# Last update:  $Id: mdadm-event-handler.sh 34650 2013-11-04 12:23:33Z hbfl $
#
# Copyright (C) 2006-2008 by Arno van Amersfoort
# Copyright (c) 2011-2013 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
#
#----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#----------------------------------------------------------------------------

CONFIG="/etc/mdadm/mdadm.conf"

MAILSEND=false
SYSLOG=false

# ---------------------------------------------------------------------------
# parse event
# ---------------------------------------------------------------------------
parse_event()
{
    echo "Host            : $HOSTNAME"
    if [ -z "${1}" ]
    then
        echo "Event           : Test message"
    else
        echo "MD Device       : ${2}"
        echo "Event           : ${1}"
        if [ -n "${3}" ]
        then
            echo "Device Component: ${3}"
        fi
    fi

    echo
    echo "/proc/mdstat dump:"
    FAIL=0
    DEGRADED=0
    MISMATCH=0
    unset IFS
    while read LINE
    do
        printf "%s" "${LINE}"
        if echo "${LINE}" | grep -q ': active '
        then
            DEV="$(echo "${LINE}" | awk '{ print $1 }')"

            printf " (mismatch_cnt=%s)" "$(cat /sys/block/$DEV/md/mismatch_cnt)"
            if [ "$(cat /sys/block/$DEV/md/mismatch_cnt)" != "0" ]
            then
                MISMATCH=$((${MISMATCH} +1))
            fi

            if echo "${LINE}" | grep -q '\(F\)'
            then
                FAIL=$((${FAIL} + 1))
                printf " (WARNING: FAILED DISK(S)!)"
            fi

            if echo "${LINE}" | grep -q '\(S\)'
            then
                printf " (Hotspare(s) available)"
            else
                printf " (NOTE: No hotspare?!)"
            fi
        fi

        if echo "${LINE}" | grep -q 'blocks'
        then
            if echo "${LINE}" | grep -q '_'
            then
                DEGRADED=$((${DEGRADED} + 1))
                printf " (DEGRADED!!!)"
            fi
        fi

        echo
    done < /proc/mdstat

    if [ ${FAIL} -gt 0 ]
    then
        echo
        echo "** WARNING: ${FAIL} MD(RAID) array(s) have FAILED disk(s)! **"
    fi

    if [ ${DEGRADED} -gt 0 ]
    then
        echo
        echo "** WARNING: ${DEGRADED} MD(RAID) array(s) are running in degraded mode! **"
    fi
}


# ---------------------------------------------------------------------------
# mail send
# ---------------------------------------------------------------------------
event_by_mail ()
{

SENDMAIL='/usr/sbin/sendmail'
MAILADDR="root"
full_domain="$(hostname -f)"


. /var/install/include/eistime
    {
    echo "From: mdadm RAID Agent <root@${full_domain}>"
    echo "To: <${MAILADDR}>"
    echo "Subject: RAID(MD) event Report from Server '${HOSTNAME}'"
    echo "Mime-Version: 1.0"
    echo "X-Mailer: sendmail mdadm on eisfair"
    echo "Content-Type: text/plain; charset=us-ascii"
    echo "Content-Transfer-Encoding: 8bit"
    echo
    echo
    echo "Dispatched from mdadm Agent on Server '${HOSTNAME}'"
    echo "Current Date: ${EISDATE} Time: ${EISTIME}"
    echo
    parse_event "${1}" "${2}" "${3}"
    } | ${SENDMAIL} ${MAILADDR}

}

# ---------------------------------------------------------------------------
# syslog
# ---------------------------------------------------------------------------
event_by_syslog ()
{

# License: GPL ver.2
# Copyright (C) 2004 SEKINE Tatsuo <tsekine@sdri.co.jp>

event="${1}"
dev="${2}"
disc="${3}"

facility="kern"
tag="mdmonitor"

case x"${event}" in
    xFail*) priority="error" ;;
    xTest*) priority="debug" ;;
    x*)     priority="info"  ;;
esac

msg="${event} event on ${dev}"
if [ x"${disc}" != x ]
then
    msg="${msg}, related to disc ${disc}"
fi

exec logger -t "${tag}" -p "${facility}.${priority}" -- "${msg}"

}

# ---------------------------------------------------------------------------
# main
# ---------------------------------------------------------------------------

# Sleep 3 second just to make sure things are in a 'stable state'
sleep 3


event="${1}"
dev="${2}"
disc="${3}"


if ${MAILSEND}
then
    event_by_mail "${event}" "${dev}" "${disc}"
fi

if ${SYSLOG}
then
    event_by_syslog "${event}" "${dev}" "${disc}"
fi


exit 0

# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
