#! /bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/mdadm-update.sh - paramater update script
#
# Creation   :  2011-10-02 hbfl
# Last update:  $Id: mdadm-update.sh 34650 2013-11-04 12:23:33Z hbfl $
#
# Copyright (c) 2011-2013 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# set package name
package_name=mdadm

# include configlib
. /var/install/include/configlib

# ---------------------------------------------------------------------------
# set default values for configuration
# ---------------------------------------------------------------------------
START_MDADM='no'

MDADM_MD_N='1'

MDADM_MD_1_DEVICE='md1'
MDADM_MD_1_ACTIVE='no'
MDADM_MD_1_LEVEL='raid1'
MDADM_MD_1_DISC_N='2'
MDADM_MD_1_DISC_1='hda1'
MDADM_MD_1_DISC_2='hdc1'

MDADM_DELAY='60'
MDADM_MAIL="root@$(hostname -d)"
MDADM_SEND_MAIL_ON_START='no'
MDADM_SEND_EVENT_TO_SYSLOG='no'
MDADM_SEND_EVENT_MAIL='no'
MDADM_SEND_STATUS_MAIL=''
# ---------------------------------------------------------------------------
# read old configuration and rename old variables
# ---------------------------------------------------------------------------
rename_old_variables()
{
    # read old values
    if [ -f /etc/config.d/${package_name} ]
    then
        /var/install/bin/backup-file --quiet ${package_name}
        . /etc/config.d/${package_name}
    fi

    md_search_file=$(find /tmp -maxdepth 1 -name "md-search.*")

    if [ -n "${md_search_file}" ]
    then
        . "${md_search_file}"
    fi
}

# ---------------------------------------------------------------------------
# Write config and default files
# ---------------------------------------------------------------------------
make_config_file()
{
    internal_conf_file=${1}
    {
    # -----------------------------------------------------------------------
    printgpl --conf ${package_name} "2011-10-02" "hbfl"
    # -----------------------------------------------------------------------

    # -----------------------------------------------------------------------
    printgroup "Basic configuration"
    # -----------------------------------------------------------------------

    printvar "START_MDADM"    "Use yes or no"
    echo
    printvar "MDADM_MD_N"     "How many MD groups are given"

    echo
    idx=1
    while [ "${idx}" -le "${MDADM_MD_N}" ]
    do
        eval md_idx='${MDADM_MD_'${idx}'_DEVICE}'

        printvar "MDADM_MD_${idx}_DEVICE" "MD device nr, like md${md_idx}"
        printvar "MDADM_MD_${idx}_ACTIVE" "Use yes or no"
        printvar "MDADM_MD_${idx}_LEVEL"  "Use RAID level, like 1"
        printvar "MDADM_MD_${idx}_DISC_N" "How many harddiscs are given"

        eval mdadm_disc='${MDADM_MD_'${idx}'_DISC_N}'

        idx2=1
        while [ "${idx2}" -le "${mdadm_disc}" ]
        do
            printvar "MDADM_MD_${idx}_DISC_${idx2}"  "Use harddisc name, like hdX${md_idx} or sdX${md_idx}"

            idx2=$(expr ${idx2} + 1 )
        done

        echo
        idx=$(expr ${idx} + 1 )
    done

    # -----------------------------------------------------------------------
    printgroup -b 0 "Global settings"
    # -----------------------------------------------------------------------

    printvar "MDADM_DELAY"                 "Polling time, default = 60 sec"
    printvar "MDADM_MAIL"                  "Use: mailaddress from admin"
    printvar "MDADM_SEND_MAIL_ON_START"    "Send email on start yes/no"
    printvar "MDADM_SEND_EVENT_TO_SYSLOG"  "Send event messages to syslog yes/no"
    printvar "MDADM_SEND_EVENT_MAIL"       "Send event messages as mail yes/no"
    printvar "MDADM_SEND_STATUS_MAIL"      "Send status messages with cron"

    # -----------------------------------------------------------------------
    printend
    # -----------------------------------------------------------------------

    } > ${internal_conf_file}
    # Set rights
    chmod 0600 ${internal_conf_file}
    chown root ${internal_conf_file}
}
# ---------------------------------------------------------------------------
# create check.d file
# ---------------------------------------------------------------------------
make_check_file()
{
printgpl --check ${package_name} '2011-10-02' 'hbfl' >/etc/check.d/${package_name}
cat >>/etc/check.d/${package_name} <<EOF
# Variable                  OPT_VARIABLE       VARIABLE_N          VALUE
START_MDADM                 -                  -                   YESNO

MDADM_MD_N                  START_MDADM        -                   NUMERIC

MDADM_MD_%_DEVICE           START_MDADM        MDADM_MD_N          MDADM_MD_CUI
MDADM_MD_%_ACTIVE           START_MDADM        MDADM_MD_N          YESNO
MDADM_MD_%_LEVEL            MDADM_MD_%_ACTIVE  MDADM_MD_N          MDADM_LEVEL_CUI
MDADM_MD_%_DISC_N           MDADM_MD_%_ACTIVE  MDADM_MD_N          NUMERIC
MDADM_MD_%_DISC_%           MDADM_MD_%_ACTIVE  MDADM_MD_%_DISC_N   PARTITION


MDADM_DELAY                 START_MDADM        -                   NUMERIC
MDADM_MAIL                  START_MDADM        -                   MAILADDR
MDADM_SEND_MAIL_ON_START    START_MDADM        -                   YESNO
MDADM_SEND_EVENT_TO_SYSLOG  START_MDADM        -                   YESNO
MDADM_SEND_EVENT_MAIL       START_MDADM        -                   YESNO
MDADM_SEND_STATUS_MAIL      START_MDADM        -                   ECRONTAB
EOF

   # Set rights for check.d file
   chmod 0600 /etc/check.d/${package_name}
   chown root /etc/check.d/${package_name}


printgpl --check_exp ${package_name} '2011-10-02' 'hbfl'  >/etc/check.d/${package_name}.exp
cat >>/etc/check.d/${package_name}.exp <<EOF
MDADM_MD_CUI      = 'md[0-9]|md10'
                  : 'Use md0,md1,md2,md3,md4,md5,md6,md7,md8,md9,md10'
MDADM_LEVEL_CUI   = 'linear|raid[01456]|raid10|multipath'
                  : 'Use linear,raid0,raid1,raid4,raid5,raid6,raid10,multipath'
EOF

    # Set rights for check.exp file
    chmod 0600 /etc/check.d/${package_name}.exp
    chown root /etc/check.d/${package_name}.exp
}
# ---------------------------------------------------------------------------
# main
# ---------------------------------------------------------------------------
# write default config file
make_config_file /etc/default.d/${package_name}

# update from old version
rename_old_variables

# write new config file
make_config_file /etc/config.d/${package_name}

# write check.d file
make_check_file

exit 0
# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
