#! /bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/mdadm.sh
#
# Creation:     2011-10-02 hbfl
# Last Update:  $Id: mdadm.sh 34650 2013-11-04 12:23:33Z hbfl $
#
# Copyright (c) 2011-2013 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib

package_name='mdadm'

# set config file
config_file="/etc/config.d/${package_name}"
. ${config_file}

mdadm_conf=/etc/${package_name}/${package_name}.conf
exec_script=/var/install/config.d/mdadm.sh
mdadm_event_handler=/sbin/mdadm-event-handler.sh
VERSION=2.2.0

#---------------------------------------------------------------------------
# start
#---------------------------------------------------------------------------
mdadm_start ()
{
    cd /etc/rc2.d
    ln -sf ../init.d/${package_name} S80${package_name}
}
#---------------------------------------------------------------------------
# stop
#---------------------------------------------------------------------------
mdadm_stop ()
{
    rm -f /etc/rc2.d/S??${package_name}
}
# ---------------------------------------------------------------------------
# create mdadm.conf
# ---------------------------------------------------------------------------
create_conf_file ()
{

cat > ${mdadm_conf} <<EOF
#----------------------------------------------------------------------------
#  mdadm.conf file generated by ${package_name} version: ${VERSION}
#
#  Do not edit this file, edit /etc/config.d/${package_name}
#  Creation date: ${EISDATE} Time: ${EISTIME}
#----------------------------------------------------------------------------
EOF


    idx=1
    while [ "${idx}" -le "${MDADM_MD_N}" ]
    do

        eval md_device='${MDADM_MD_'${idx}'_DEVICE}'
        eval md_active='${MDADM_MD_'${idx}'_ACTIVE}'
        eval  md_level='${MDADM_MD_'${idx}'_LEVEL}'

        if [ "${md_active}" = "yes" ]
        then

            eval md_disc_n='${MDADM_MD_'${idx}'_DISC_N}'
            _disc=''

            idx2=1
            while [ "${idx2}" -le "${md_disc_n}" ]
            do

                eval md_disc='${MDADM_MD_'${idx}'_DISC_'${idx2}'}'

                case ${md_disc} in
                    sd*|hd*)
                        raw_disc=$(echo "${md_disc}" | sed 's/\(...\).$/\1/')

                        if ! grep -q "${raw_disc}" ${mdadm_conf}
                        then
                            echo "DEVICE /dev/${raw_disc}[1-9]" >>${mdadm_conf}
                        fi
                        ;;
                    c0d*)
                        raw_disc=$(echo "${md_disc}" | sed 's/\(....\).$/\1/')

                        if ! grep -q "${raw_disc}" ${mdadm_conf}
                        then
                            echo "DEVICE /dev/${raw_disc}[1-9]" >>${mdadm_conf}
                        fi
                        ;;
                esac

                if [ -z "${_disc}" ]
                then
                    _disc="/dev/${md_disc}"
                else
                    _disc="${_disc},/dev/${md_disc}"
                fi

                idx2=$(expr ${idx2} + 1 )
            done

            echo "ARRAY /dev/${md_device} level=${md_level} devices=${_disc}" >>${mdadm_conf}
        fi

        idx=$(expr ${idx} + 1 )
    done

    echo "PROGRAM ${mdadm_event_handler}" >>${mdadm_conf}
}

# ---------------------------------------------------------------------------
# push info
# ---------------------------------------------------------------------------
push_info ()
{
    md_mail="${MDADM_MAIL}"

    if [ "${MDADM_SEND_EVENT_TO_SYSLOG}" = "yes" ]
    then
        md_syslog="true"
    fi

    if [ "${MDADM_SEND_EVENT_MAIL}" = "yes" ]
    then
        md_send="true"
    fi

    sed -i "s/^MAILADDR=.*$/MAILADDR='${md_mail}'/;
            s/^MAILSEND=.*$/MAILSEND='${md_send:-false}'/;
            s/^SYSLOG=.*$/SYSLOG='${md_syslog:-false}'/" \
            /sbin/mdadm-event-handler.sh
}

# ---------------------------------------------------------------------------
# check sendmail
# ---------------------------------------------------------------------------
check_sendmail ()
{
    if [ "${MDADM_SEND_EVENT_MAIL}" = "yes" \
      -o "${MDADM_SEND_MAIL_ON_START}" = "yes" ]
    then
        if [ ! -f '/usr/sbin/sendmail' ]
        then
            set_parameter_no ${MDADM_SEND_EVENT_MAIL}
            set_parameter_no ${MDADM_SEND_MAIL_ON_START}

            echo
            mecho --error "'smtp' service not found,"
            mecho --error "the event mail can not be send."
            mecho --info  "Please install a 'smtp' service first."
            echo
            anykey
        fi
    fi
}

# ---------------------------------------------------------------------------
# set parameter to no
# ---------------------------------------------------------------------------
set_parameter_no ()
{
    # set parameter from 'yes' to 'no'
    sed -i "s/^${1}=.*/${1}='no'/" ${config_file}

    echo "${1}='no'" >/tmp/eis-conf.$$
    . /tmp/eis-conf.$$
    rm -f /tmp/eis-conf.$$
}

# ---------------------------------------------------------------------------
# create crontab
# ---------------------------------------------------------------------------
create_crontab ()
{
    if [ -n "${MDADM_SEND_STATUS_MAIL}" ]
    then
        # create cron job
        cat >/var/cron/etc/root/mdadm <<EOF
#----------------------------------------------------------------------------
#  cron mdadm file generated by ${package_name} version: ${VERSION}
#
#  Do not edit this file, edit ${config_file}
#  Creation Date: ${EISDATE} Time: ${EISTIME}
#----------------------------------------------------------------------------
${MDADM_SEND_STATUS_MAIL} ${exec_script} make-status
EOF

        # update crontab file
        /var/install/config.d/cron >/dev/null 2>&1
    else
        # when no, delete the cron job
        rm -rf /var/cron/etc/root/mdadm

        # update crontab file
        /var/install/config.d/cron >/dev/null 2>&1
    fi
}

# ---------------------------------------------------------------------------
# do status
# ---------------------------------------------------------------------------
do_status ()
{
    ${mdadm_event_handler} "Status message"

}

# ---------------------------------------------------------------------------
# remove crontab
# ---------------------------------------------------------------------------
remove_crontab ()
{
    # when no, delete the cron job
    rm -rf /var/cron/etc/root/mdadm

    # update crontab file
    /var/install/config.d/cron >/dev/null 2>&1
}
# ---------------------------------------------------------------------------
# main
# ---------------------------------------------------------------------------

    case ${1} in
    make-status)
        do_status
        ;;
    *)
        case ${START_MDADM} in
        yes)
            mdadm_start
            mkdir -p /etc/mdadm

            create_conf_file
            check_sendmail
            push_info
            create_crontab
            ;;
        no)
            mdadm_stop
            remove_crontab
            ;;
        esac
        ;;
    esac

exit 0
# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
